package com.hai.openApi.config;

import com.hai.entity.ApiAmountRecord;
import com.hai.entity.ApiMerchants;
import com.hai.entity.ApiOrder;
import com.hai.entity.HighOrder;
import com.hai.openApi.service.ApiOrderService;
import com.hai.order.type.OrderProductType;
import com.hai.service.ApiAmountRecordService;
import com.hai.service.ApiMerchantsService;
import com.hai.service.HighOrderService;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Date;

@Component
public class PriceComputeConfig {

    @Resource
    private ApiMerchantsService apiMerchantsService;

    @Resource
    private ApiAmountRecordService apiAmountRecordService;

    @Resource
    private HighOrderService highOrderService;

    @Resource
    private ApiOrderService apiOrderService;

    public  Boolean mchBalance(ApiMerchants apiMerchants , BigDecimal price , String orderNo) {


        // 当前账号余额是否可以充值当前金额
        if (apiMerchants.getAmounts().compareTo(price) < 0) {
            return false;
        }

        ApiOrder apiOrder = apiOrderService.findByOrderNo(orderNo , null);

        HighOrder order = highOrderService.getOrderByOrderNo(apiOrder.getOrderNo());

        // 插入金额记录
        // 变更前金额
        BigDecimal beforeAmount = apiMerchants.getAmounts();
        // 计算金额
        apiMerchants.setAmounts(apiMerchants.getAmounts().  subtract(price));
        // 变更后金额
        BigDecimal afterAmount = apiMerchants.getAmounts();

        apiMerchantsService.updateApiMerchants(apiMerchants);

        ApiAmountRecord apiAmountRecord = new ApiAmountRecord();

        apiAmountRecord.setCreateTime(new Date());
        apiAmountRecord.setUpdateTime(new Date());
        apiAmountRecord.setMchId(apiMerchants.getMchId());

        apiAmountRecord.setStatus(100);
        apiAmountRecord.setAmount(price);
        apiAmountRecord.setAfterAmount(afterAmount);
        apiAmountRecord.setBeforeAmount(beforeAmount);
        apiAmountRecord.setAmountType(2);
        apiAmountRecord.setSourceType(OrderProductType.getDataByType(order.getProductType()).getNumber());
        apiAmountRecord.setSourceOrderNo(orderNo);
        apiAmountRecord.setSourceId(apiMerchants.getId());
        apiAmountRecord.setSourceContent(apiMerchants.getMerchantName() + "|" + OrderProductType.getDataByType(order.getProductType()).getName() + "｜扣款" + price.setScale(2, BigDecimal.ROUND_HALF_DOWN));

        apiAmountRecordService.insertAmountRecord(apiAmountRecord);


        return true;
    }

    /**
     * @Author Sum1Dream
     * @Name computationPriceCost
     * @Description // 计算折扣价格
     * @Date 09:25 2023/3/13
     * @Param discount 折扣
     * @Param price 原价
     * @Return java.math.BigDecimal
     */
    public BigDecimal computationPriceCost(BigDecimal discount , BigDecimal price) {
        return price.multiply(discount).divide(new BigDecimal(100)).setScale(2, BigDecimal.ROUND_HALF_UP);

    }

    public BigDecimal computationPriceCost(BigDecimal discount , BigDecimal price , BigDecimal costPrice) {
        return  costPrice.add(price.multiply(discount.divide(new BigDecimal(100)))).setScale(2, RoundingMode.HALF_UP);

    }
}
